@@ -1,14 +1,45 @@
 package Pugs::Emitter::Grammar::Perl5;
 
-our $VERSION = '0.25';
+our $VERSION = '0.27';
 
 #use Smart::Comments;
 use strict;
 use warnings;
 use Pugs::Emitter::Rule::Perl5::Ratchet;
 
+# for safe mode
+sub _prune_actions {
+    my ($ast) = @_;
+    while (my ($key, $node) = each %$ast) {
+        next if $key =~ /^_/ or !ref $node;
+        #warn $key;
+        if ($key eq 'closure') {
+            #die "Found closures!";
+            next if ref $node ne 'HASH';
+            my $code = $node->{closure};
+            if ($code and !ref $code and $code =~ /\w+/) {
+                die "ERROR: code blocks not allowed in safe mode: \"$code\"\n";
+            }
+        }
+        if (ref $node) {
+            my $ref = ref $node;
+            if ($ref eq 'HASH') {
+                _prune_actions($node); 
+            } elsif ($ref eq 'ARRAY') {
+                for my $child (@$node) {
+                    if (ref $child and ref $child eq 'HASH') {
+                        _prune_actions($child);
+                    }
+                }
+            }
+        }
+    }
+}
+
 sub emit {
     my $ast = shift;
+    my $opts = shift;
+    $opts ||= {};
     ## $ast
     my ($name, $stmts) = each %$ast;
     my $p5_methods = '';
@@ -18,9 +49,13 @@ sub emit {
         my $type = $regex->{type};
         ## $regex
         if ($type eq 'block') {
+            my $code = $regex->{value};
+            if ($opts->{safe_mode} && $code =~ /\w+/) {
+                die "ERROR: verbatim Perl 5 blocks not allowed in safe mode: \"$code\"\n";
+            }
             $p5_methods .= <<"_EOC_";
 # Code block from grammar spec
-$regex->{value}
+$code
 
 _EOC_
             next;
@@ -32,15 +67,21 @@ _EOC_
             $params->{sigspace} = 1;
         }
         my $body;
+
+        my $ast = $regex->{ast};
+        if ($opts->{safe_mode}) {
+            _prune_actions($ast);
+        }
+
         if ($type eq 'regex') {
             $body = Pugs::Emitter::Rule::Perl5::emit(
                 'Pugs::Grammar::Rule',
-                $regex->{ast},
+                $ast,
             )
         } else {
             $body = Pugs::Emitter::Rule::Perl5::Ratchet::emit(
                 'Pugs::Grammar::Rule',
-                $regex->{ast},
+                $ast,
                 $params,
             );
         }
@@ -62,6 +103,7 @@ ${prefix}use base 'Pugs::Grammar::Base';
 
 use Pugs::Runtime::Match;
 use Pugs::Runtime::Regex;
+use Pugs::Runtime::Tracer ();
 
 $p5_methods
 
@@ -114,7 +156,7 @@ The Pugs contributors E<lt>perl6-compiler@perl.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007 by Agent Zhang and others.
+Copyright (c) 2007 by Agent Zhang (E<lt>agentzh@agentzh.orgE<gt>) and others.
 
 This program is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
